# global parameters for simulation runs: 
# number of simulated trials nsim (20,000), sample size sampsi (30,50,80,100,150), 
# pth and qth quantiles 0.025 and 0.975
nsim<-20000
sampsi<-50
p<-0.025
q<-1-p

# This function calculates the coverage probability for the nonparametric Limits of Agreement on samples from six distributions 
# (Standard Normal distribution; Normal distribution with 1,2 and 5% outliers; Exponential distribution; and Lognormal distribution).
# The function takes the sample size n, a possible subsample size k, a possible bandwidth h and a quantile estimator as input.
coverageestimate<-function(n,k,h,estimator){
  # 6 Matrices on (nsim x (n+1)) are generated. They consist of data for the six distributions. The distributions are the normal distribution,
  #the normal distribution with 1, 2 and 5% outliers, the exponentiel distribution and the lognormal distribution.
  set.seed(256)
  D1=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D2=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D3=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D4=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D5=matrix(rexp((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D6=matrix(rlnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  for(row in 1:nsim) {
    for(col in 1:(n+1)) {
      #generation of outliers to the Standard Normal distribution; first a random number between 0 and 1 is generated
      tal<-runif(1,0,1)
      # If the random number is smaller than 0.01, then the value generated from the Standard Normal distribution is replaced by a number from 
      # the Normal distribution with mean 0 and standard deviation 3.In this way, a Normal distribution with 1% outliers is generated.
      if (tal<0.01) {
        D2[row,col]=rnorm(1,0,3)
      }
      # Similar procedure for a Normal distribution with 2% outliers
      if (tal<0.02) {
        D3[row,col]=rnorm(1,0,3)
      }
      # Similar procedure for a Normal distribution with 5% outliers
      if (tal<0.05) {
        D4[row,col]=rnorm(1,0,3)
      }
    }
  }
  
  # The (n+1) observations are removed
  D1n<-D1[,-(n+1)]
  D2n<-D2[,-(n+1)]
  D3n<-D3[,-(n+1)]
  D4n<-D4[,-(n+1)]
  D5n<-D5[,-(n+1)]
  D6n<-D6[,-(n+1)]
  
  # 6 new matrices are created for the 6 different distributions
  E1<-matrix(NA,nsim,4,byrow=TRUE)
  E2<-matrix(NA,nsim,4,byrow=TRUE)
  E3<-matrix(NA,nsim,4,byrow=TRUE)
  E4<-matrix(NA,nsim,4,byrow=TRUE)
  E5<-matrix(NA,nsim,4,byrow=TRUE)
  E6<-matrix(NA,nsim,4,byrow=TRUE)
  # The first column in the first matrix consists of the (n+1) observations from the Standard Normal distribution
  E1[,1]<-D1[,(n+1)]
  # The next two columns in the first matrix are the upper and lower nonparametric Limits of Agreement for a chosen quantile estimator
  E1[,2]<-apply(D1n,1,estimator,k=k[1],h=h[1],n=n,p=p)
  E1[,3]<-apply(D1n,1,estimator,k=k[2],h=h[2],n=n,p=q)
  # The first column in the second matrix consists of the (n+1) observations for a Normal distribution with 1% outliers
  E2[,1]<-D2[,(n+1)]
  # The next two columns in the second matrix are the upper and lower nonparametric Limits of Agreement for a chosen quantile estimator
  E2[,2]<-apply(D2n,1,estimator,k=k[3],h=h[3],n=n,p=p)
  E2[,3]<-apply(D2n,1,estimator,k=k[4],h=h[4],n=n,p=q)
  # The first column in the third matrix consists of the (n+1) observations for a Normal distribution with 2% outliers
  E3[,1]<-D3[,(n+1)]
  # The next two columns in the third matrix are the upper and lower nonparametric Limits of Agreement for a chosen quantile estimator
  E3[,2]<-apply(D3n,1,estimator,k=k[5],h=h[5],n=n,p=p)
  E3[,3]<-apply(D3n,1,estimator,k=k[6],h=h[6],n=n,p=q)
  # The first column in the fourth matrix consists of the (n+1) observations for a Normal distribution with 5% outliers
  E4[,1]<-D4[,(n+1)]
  # The next two columns in the fourth matrix are the upper and lower nonparametric Limits of Agreement for a chosen quantile estimator
  E4[,2]<-apply(D4n,1,estimator,k=k[7],h=h[7],n=n,p=p)
  E4[,3]<-apply(D4n,1,estimator,k=k[8],h=h[8],n=n,p=q)
  # The first column in the fifth matrix consists of the (n+1) observations for the Exponential distribution.
  E5[,1]<-D5[,(n+1)]
  # The next two columns in the fifth matrix are the upper and lower nonparametric Limits of Agreement for a chosen quantile estimator
  E5[,2]<-apply(D5n,1,estimator,k=k[9],h=h[9],n=n,p=p)
  E5[,3]<-apply(D5n,1,estimator,k=k[10],h=h[10],n=n,p=q)
  # The first column in the sixth matrix consists of the (n+1) observations for the Lognormal distribution.
  E6[,1]<-D6[,(n+1)]
  # The next two columns in the sixth matrix are the upper and lower nonparametric Limits of Agreement for a chosen quantile estimator
  E6[,2]<-apply(D6n,1,estimator,k=k[11],h=h[11],n=n,p=p)
  E6[,3]<-apply(D6n,1,estimator,k=k[12],h=h[12],n=n,p=q)
  # If the (n+1) observation is between the Limits of Agreement, the fourth column is set to 1; else it is set to 0.
  # This is done for all six distributions.
  for (i in 1:nsim){
    if (E1[i,2]<=E1[i,1] & E1[i,1]<=E1[i,3]){
      E1[i,4]<-1
    }else{
      E1[i,4]<-0
    }
    if (E2[i,2]<=E2[i,1] & E2[i,1]<=E2[i,3]){
      E2[i,4]<-1
    }else{
      E2[i,4]<-0
    }
    if (E3[i,2]<=E3[i,1] & E3[i,1]<=E3[i,3]){
      E3[i,4]<-1
    }else{
      E3[i,4]<-0
    }
    if (E4[i,2]<=E4[i,1] & E4[i,1]<=E4[i,3]){
      E4[i,4]<-1
    }else{
      E4[i,4]<-0
    }
    if (E5[i,2]<=E5[i,1] & E5[i,1]<=E5[i,3]){
      E5[i,4]<-1
      }else{
        E5[i,4]<-0
        }
    if (E6[i,2]<=E6[i,1] & E6[i,1]<=E6[i,3]){
        E6[i,4]<-1
      }else{
        E6[i,4]<-0
      }
    }
  
  #The coverage probability is estimated for each distribution
  Coveragematrix<-matrix(NA,1,6,byrow=TRUE);
  Coveragematrix[1,1]<-sum(E1[,4])/nsim
  Coveragematrix[1,2]<-sum(E2[,4])/nsim
  Coveragematrix[1,3]<-sum(E3[,4])/nsim
  Coveragematrix[1,4]<-sum(E4[,4])/nsim
  Coveragematrix[1,5]<-sum(E5[,4])/nsim
  Coveragematrix[1,6]<-sum(E6[,4])/nsim
  return(Coveragematrix)
}

# (1) Sample quantile estimator SQp1; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA and h=NA
samplequantile1<-function(k,h,n,p,x){
  s<-n*p
  x1<-sort(x)
  #The SQp1 estimate is calculated and returned 
  if(floor(s)==s){
    Qp<-x1[s]
  } else{
    Qp<-x1[floor(s)+1]
  }
  return(Qp)
}

# (2) Sample quantile estimator SQp2; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA and h=NA
samplequantile2<-function(k,h,n,p,x){
  s<-p*(n+1)
  r<-floor(p*(n+1))
  alpha<-s-r
  x1<-sort(x)
  #The SQp2 estimate is calculated and returned 
  Qp<-(1-alpha)*x1[r]+alpha*x1[r+1]
  return(Qp)
}

# (3) Sample quantile estimator SQIp; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA and h=NA
samplequantileinterpolation<-function(k,h,n,p,x){
  alpha<-0.5
  beta<-0.5
  m<-alpha+p*(1-alpha-beta)
  s<-(n*p)+m
  s1<-floor((n*p)+m)
  gamma<-s-s1
  x1<-sort(x)
  #The SQIp estimate is calculated and returned
  est<-(1-gamma)*x1[s1]+gamma*x1[s1+1]
  return(est)
}

# (4) Subsampling quantile estimator HDp; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA and h=NA
HD<-function(k,h,n,p,x){
  s1<-p*(n+1)
  s2<-(1-p)*(n+1)
  x1<-sort(x)
  #The HDp estimate is calculated and returned
  W1<-pbeta(c(1:n)/n, s1, s2)-pbeta((c(1:n)-1)/n,s1,s2)
  est<-t(W1)%*%x1
  return(est)
}

# (5) Subsampling quantile estimator KLp; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, h=NA
KL<-function(k,h,n,p,x){
  r<-floor((k+1)*p)
  nkbinom<-choose(n,k)
  #r must be larger than 0 for the KL estimator to be a valid estimator
  if(r==0){
    stop('r must be larger larger than 0')
  }else{
    #the Weights for the KL estimator are calculated
    jrbinom<-choose(c(r:(n+r-k))-1,r-1)
    njkrbinom<-choose(n-c(r:(n+r-k)),k-r)
    weight<-(jrbinom*njkrbinom)/nkbinom
    x1<-sort(x)
    # The KLp estimate is calculated and returned
    est1<-t(weight)%*%x1[c(r:(n+r-k))]
  }
  return(est1)
}

# (6) Subsampling quantile estimator KCp; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, h=NA
kaighcheng<-function(k,h,n,p,x){
  r<-ceiling(k*p)
  #The weights for the KCp estimator are calculated
  weight<-choose(r+c(1:n)-2,r-1)*choose(n-c(1:n)+k-r,k-r)/choose(n+k-1,k)
  x1<-sort(x)
  #The KCp estimate is calculated and returned
  est<-t(weight)%*%x1
  return(est)
}

# (7) Subsampling quantile estimator BPp; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA h=NA
bernstein<-function(k,h,n,p,x){
    x1<-sort(x)
    weight<-choose(n-1,c(1:n)-1)*p^(c(1:n)-1)*(1-p)^(n-c(1:n))
    #The BPp estimate is calculated and returned
    est<-t(weight)%*%x1
  return(est)
}

# (8) Kernel quantile estimator KQp1; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA
kernelest1<-function(k,h,n,p,x){
  #The Gaussian kernel is used
  ker<-function(u){(1/(sqrt(2*pi)))*exp(-0.5*u^2)}
  #The weights of the KQp1 estimator are calculated
  u<-ker((c(1:n)/n - p ) / h )
  x1<-sort(x)
  #The KQp1 estimate is calculated and returned
  est<-mean(x1*u) /h
  return(est)
}

# (9) Kernel quantile estimator KQp2; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA
kernelest2<- function(k,h,n,p,x) {
  #The Gaussian kernel is used
  ker<-function(u){(1/(sqrt(2*pi)))*exp(-0.5*u^2)}
  u<-ker(((c(1:n)-0.5)/n - p ) / h)
  x1<-sort(x)
  #The KQp2 estimate is calculated and returned
  est<-(sum(x1*u)/h)/(sum(u)/h)
  return(est)
}

# WEIGHT FUNCTION used for the KQplc and the HDplc estimator
wei<-function(i,n){
  ifelse((i==1) |(i==n), 0.5*(1-((n-2)/(sqrt(n*(n-1))))), 1/(sqrt(n*(n-1))))
}

# (10) Kernel quantile estimator KQplc; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA
kernellevelcross<-function(k,h,n,p,x){
  #The Gaussian kernel is used
  ker<-function(u){(1/(sqrt(2*pi)))*exp(-0.5*u^2)}
  #Weights
  q1<-wei((c(1:n)),n)
  q1<-cumsum(q1)
  #The KQplc estimate is calculated and returned
  u<-ker((q1-p) / h )
  x1<-sort(x)
  est<-mean(u*x1)/h
  return(est)
}

# (11) Kernel quantile estimator HDplc; the function takes a subsample size of k, a bandwidth h, a sample size n and a sample x as input.
# Here, h=NA, k=NA
HDlevelcross<-function(k,h,n,p,x){
  s1<-p*(n+1)
  s2<-(1-p)*(n+1)
  q1<-wei((c(1:n)),n)
  q1<-cumsum(q1)
  x1<-sort(x)
  #The HDplc estimate is calculated and returned
  first<-(pbeta(q1[1],s1,s2)-pbeta(0,s1,s2))*x1[1]
  est<-sum((pbeta(q1[2:n],s1,s2)-pbeta(q1[(2:n)-1],s1,s2))*x1[2:n])
  est<-first+est
  return(est)
}


# (12) Sfakianakis-Verginis type estimator SVp1; the function takes a subsample size of k, a bandiwdth h, a sample size n and a sample x as input.
# Here, k=NA, h=NA
SV1quantile<-function(k,h,n,p,x){
    x1<-sort(x)
    #The SVp1 estimate is calculated and returned
    first<-((2*dbinom(0,n,p)+dbinom(1,n,p))/2)*x1[1]+(dbinom(0,n,p)/2)*x1[2]-(dbinom(0,n,p)/2)*x1[3]
    last<-(-dbinom(n,n,p)/2)*x1[n-2]+(dbinom(n,n,p)/2)*x1[n-1]+((2*dbinom(n,n,p)+dbinom((n-1),n,p))/2)*x1[n]
    est<-sum(((dbinom((2:(n-1)),n,p)+dbinom(((2:(n-1))-1),n,p))/2)*x1[(2:(n-1))])
    est<-first+est+last
  return(est)
}

# (13) Sfakianakis-Verginis type estimator SVp2; the function takes a subsample of size k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA, h=NA
SV2quantile<-function(k,h,n,p,x){
    x1<-sort(x)
    #The SVp2 estimate is calculated and returned
    est<-sum(dbinom(c(0:(n-1)),n,p)*x1[c(1:n)])+(2*x1[n]-x1[n-1])*dbinom(n,n,p)
  return(est)
}

# (14) Sfakianakis-Verginis type estimator SVp3; the function takes a subsample size of k, a bandwidth h, a sample size n and a sample x as input.
# Here, k=NA, h=NA
SV3quantile<-function(k,h,n,p,x){
    x1<-sort(x)
    # The SV3 estimate is calculated and returned
    est<-sum(dbinom(1:n,n,p)%*%x1[1:n])+(2*x1[1]-x1[2])*dbinom(0,n,p)
  return(est)
}


# (15) NO estimator; the function takes a subsample size k, a bandiwdth h, a sample size n and a sample x as input.
# Here, k=NA, h=NA
NOquantile<-function(k,h,n,p,x){
  #The sample is sorted
  x1<-sort(x)
  #The NO estimate is calculated and returned
  first<-(dbinom(0,n,p)*2*p+dbinom(1,n,p)*p)*x1[1]+dbinom(0,n,p)*(2-3*p)*x1[2]-dbinom(0,n,p)*(1-p)*x1[3]
  last<-dbinom(n,n,p)*(3*p-1)*x1[n-1]+(dbinom(n-1,n,p)*(1-p)+dbinom(n,n,p)*(2-2*p))*x1[n]
  est<-sum((dbinom((1:(n-2)),n,p)*(1-p)+dbinom(((2:(n-1))-1),n,p)*p)*x1[(2:(n-1))])-dbinom(n,n,p)*p*x1[n-2]
  est<-first+est+last
  return(est)
}




#Generation of samples for the 6 distributions to be used in calculation of values for k and h; the function takes the sample size n as input.
samplefun<-function(n){
  set.seed(256)
  D1=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D2=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D3=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D4=matrix(rnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D5=matrix(rexp((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  set.seed(256)
  D6=matrix(rlnorm((n+1)*nsim), nrow = nsim, ncol = (n+1),byrow=TRUE)
  for(row in 1:nsim) {
    for(col in 1:(n+1)) {
      tal<-runif(1,0,1)
      if (tal<0.01) {
        D2[row,col]=rnorm(1,0,3)
      }
      if (tal<0.02) {
        D3[row,col]=rnorm(1,0,3)
      }
      if (tal<0.05) {
        D4[row,col]=rnorm(1,0,3)
      }
    }
  }
  D1n<-D1[,-(n+1)]
  D2n<-D2[,-(n+1)]
  D3n<-D3[,-(n+1)]
  D4n<-D4[,-(n+1)]
  D5n<-D5[,-(n+1)]
  D6n<-D6[,-(n+1)]
  return(list("D1n"=D1n,"D2n"=D2n,"D3n"=D3n,"D4n"=D4n,"D5n"=D5n,"D6n"=D6n ))
}








#Finding the best subsample k for the KL estimator; the function takes the sample size n, the p-th quantile p,a sample y and the true quantile tq as input.
findk_KL<-function(n,p,y,tq){
  #all possible k-values are considered, and r is calculated for the k-values
  k<-c(1:n)
  r1<-floor((k+1)*p)
  #k-values, which give r-values larger than 0, are chosen
  kp<-k[r1>=1]
  #the KLp estimator is applied on the possible k-values
  out1<-matrix(NA,nsim,length(kp))
  for (j in 1:length(kp)){
    out1[,j]<-apply(y,1,KL,k=kp[j],h=NA,n=n,p=p)
  }
 
  #The MSE is calculated and the k-value implicating the smallest MSE is found and returned as the best k
  mse1<-apply((out1-tq)^2,2,mean)
  k1<-kp[which(mse1==min(mse1))]
  return(k1)
}

# Finding the best subsample k for the KCp estimator; the function takes the sample size n, the p-th quantile p,a sample y and the true quantile tq.
findk_KC<-function(n,p,y,tq){
  #all possible k-values are considered, and r is calculated for the k-values
  k<-c(1:n)
  r1<-ceiling(k*p)
  #r must be larger than 0. The k-values satisfying this condition are used
  kp<-k[r1>=1]
  #The KCp estimator is applied on the possible k-values
  out1<-matrix(NA,nsim,length(kp))
  for (j in 1:length(kp)){
    out1[,j]<-apply(y,1,kaighcheng,k=kp[j],h=NA,n=n,p=p)
  }
  #The MSE is calculated and the k-value implicating the smallest MSE is found and returned
  mse1<-apply((out1-tq)^2,2,mean)
  k1<-kp[which(mse1==min(mse1))]
  return(k1)
}

# Finding the best h-value for the kernel quantile estimators; the function takes the sample size n, the p-th quantile p, a sample y and the true quantile tq as input.
findbesth<-function(n,p,y,estimator,tq){ 
  #a possible sequence of h-values
  h<-seq(0.001,0.99,0.001)
  #a kernel quantile estimator is applied on all h-values and a matrix is created in which each column corresponds to the estimate for a h-value
  out1<-matrix(NA,nsim,length(h))
  for (j in 1:length(h)){
    out1[,j]<-apply(y,1,estimator,k=NA,h=h[j],n=n,p=p)
  }
  #The MSE is calculated and the h-value implicating the smallest MSE is returned and used
  mse1<-apply((out1-tq)^2,2,mean)
  hbest<-h[which(mse1==min(mse1))]
  return(hbest)
}

# The samples from the 6 distributions are derived.
D1n<-samplefun(sampsi)[1]
D1n<-unname(D1n)
D1n<-unlist(D1n)
D1n<-matrix(D1n,nsim,sampsi)
D2n<-samplefun(sampsi)[2]
D2n<-unname(D2n)
D2n<-unlist(D2n)
D2n<-matrix(D2n,nsim,sampsi)
D3n<-samplefun(sampsi)[3]
D3n<-unname(D3n)
D3n<-unlist(D3n)
D3n<-matrix(D3n,nsim,sampsi)
D4n<-samplefun(sampsi)[4]
D4n<-unname(D4n)
D4n<-unlist(D4n)
D4n<-matrix(D4n,nsim,sampsi)
D5n<-samplefun(sampsi)[5]
D5n<-unname(D5n)
D5n<-unlist(D5n)
D5n<-matrix(D5n,nsim,sampsi)
D6n<-samplefun(sampsi)[6]
D6n<-unname(D6n)
D6n<-unlist(D6n)
D6n<-matrix(D6n,nsim,sampsi)

# The coverage probabilities are calculated for the SQp1, SQp2, SQIp and HDp estimator
coverageestimate(sampsi,k=NA,h=NA,estimator=samplequantile1)
coverageestimate(sampsi,k=NA,h=NA,estimator=samplequantile2)
coverageestimate(sampsi,k=NA,h=NA,estimator=samplequantileinterpolation)
coverageestimate(sampsi,k=NA,h=NA,HD)

# The best k-values are found for the 6 distributions, and the coverage probability is found for the KLp estimator
kp1_KL<-findk_KL(sampsi,p,D1n,qnorm(p))
kq1_KL<-findk_KL(sampsi,q,D1n,qnorm(q))
kp2_KL<-findk_KL(sampsi,p,D2n,qnorm(p))
kq2_KL<-findk_KL(sampsi,q,D2n,qnorm(q))
kp3_KL<-findk_KL(sampsi,p,D3n,qnorm(p))
kq3_KL<-findk_KL(sampsi,q,D3n,qnorm(q))
kp4_KL<-findk_KL(sampsi,p,D4n,qnorm(p))
kq4_KL<-findk_KL(sampsi,q,D4n,qnorm(q))
kp5_KL<-findk_KL(sampsi,p,D5n,qexp(p))
kq5_KL<-findk_KL(sampsi,q,D5n,qexp(q))
kp6_KL<-findk_KL(sampsi,p,D6n,qlnorm(p))
kq6_KL<-findk_KL(sampsi,q,D6n,qlnorm(q))
k<-c(kp1_KL,kq1_KL,kp2_KL,kq2_KL,kp3_KL,kq3_KL,kp4_KL,kq4_KL,kp5_KL,kq5_KL,kp6_KL,kq6_KL)
coverageestimate(sampsi,k,h=NA,KL)

# The best k-values are found for the 6 distributions, and the coverage probability is found for the KCp estimator
kp1_KC<-findk_KC(sampsi,p,D1n,qnorm(p))
kq1_KC<-findk_KC(sampsi,q,D1n,qnorm(q))
kp2_KC<-findk_KC(sampsi,p,D2n,qnorm(p))
kq2_KC<-findk_KC(sampsi,q,D2n,qnorm(q))
kp3_KC<-findk_KC(sampsi,p,D3n,qnorm(p))
kq3_KC<-findk_KC(sampsi,q,D3n,qnorm(q))
kp4_KC<-findk_KC(sampsi,p,D4n,qnorm(p))
kq4_KC<-findk_KC(sampsi,q,D4n,qnorm(q))
kp5_KC<-findk_KC(sampsi,p,D5n,qexp(p))
kq5_KC<-findk_KC(sampsi,q,D5n,qexp(q))
kp6_KC<-findk_KC(sampsi,p,D6n,qlnorm(p))
kq6_KC<-findk_KC(sampsi,q,D6n,qlnorm(q))
k<-c(kp1_KC,kq1_KC,kp2_KC,kq2_KC,kp3_KC,kq3_KC,kp4_KC,kq4_KC,kp5_KC,kq5_KC,kp6_KC,kq6_KC)
coverageestimate(sampsi,k,h=NA,kaighcheng)

#The covarge probability is found the the BPp estimator
coverageestimate(sampsi,k=NA,h=NA,estimator=bernstein)

#Long running time for the kernel quantile estimators because of the calculation of the bandwidth h!
# The best h-values are found for the 6 distributions, and the coverage probability is found for the KQp1 estimator
hp1_KQ1<-findbesth(sampsi,p,D1n,kernelest1,qnorm(p))
hq1_KQ1<-findbesth(sampsi,q,D1n,kernelest1,qnorm(q))
hp2_KQ1<-findbesth(sampsi,p,D2n,kernelest1,qnorm(p))
hq2_KQ1<-findbesth(sampsi,q,D2n,kernelest1,qnorm(q))
hp3_KQ1<-findbesth(sampsi,p,D3n,kernelest1,qnorm(p))
hq3_KQ1<-findbesth(sampsi,q,D3n,kernelest1,qnorm(q))
hp4_KQ1<-findbesth(sampsi,p,D4n,kernelest1,qnorm(p))
hq4_KQ1<-findbesth(sampsi,q,D4n,kernelest1,qnorm(q))
hp5_KQ1<-findbesth(sampsi,p,D5n,kernelest1,qexp(p))
hq5_KQ1<-findbesth(sampsi,q,D5n,kernelest1,qexp(q))
hp6_KQ1<-findbesth(sampsi,p,D6n,kernelest1,qlnorm(p))
hq6_KQ1<-findbesth(sampsi,q,D6n,kernelest1,qlnorm(q))
h_kq1<-c(hp1_KQ1,hq1_KQ1,hp2_KQ1,hq2_KQ1,hp3_KQ1,hq3_KQ1,hp4_KQ1,hq4_KQ1,hp5_KQ1,hq5_KQ1,hp6_KQ1,hq6_KQ1)
coverageestimate(sampsi,k=NA,h_kq1,estimator=kernelest1)

#Long running time!
# The best h-values are found for the 6 distributions, and the coverage probability is found for the KQp2 estimator
hp1_KQ2<-findbesth(sampsi,p,D1n,kernelest2,qnorm(p))
hq1_KQ2<-findbesth(sampsi,q,D1n,kernelest2,qnorm(q))
hp2_KQ2<-findbesth(sampsi,p,D2n,kernelest2,qnorm(p))
hq2_KQ2<-findbesth(sampsi,q,D2n,kernelest2,qnorm(q))
hp3_KQ2<-findbesth(sampsi,p,D3n,kernelest2,qnorm(p))
hq3_KQ2<-findbesth(sampsi,q,D3n,kernelest2,qnorm(q))
hp4_KQ2<-findbesth(sampsi,p,D4n,kernelest2,qnorm(p))
hq4_KQ2<-findbesth(sampsi,q,D4n,kernelest2,qnorm(q))
hp5_KQ2<-findbesth(sampsi,p,D5n,kernelest2,qexp(p))
hq5_KQ2<-findbesth(sampsi,q,D5n,kernelest2,qexp(q))
hp6_KQ2<-findbesth(sampsi,p,D6n,kernelest2,qlnorm(p))
hq6_KQ2<-findbesth(sampsi,q,D6n,kernelest2,qlnorm(q))
h_kq2<-c(hp1_KQ2,hq1_KQ2,hp2_KQ2,hq2_KQ2,hp3_KQ2,hq3_KQ2,hp4_KQ2,hq4_KQ2,hp5_KQ2,hq5_KQ2,hp6_KQ2,hq6_KQ2)
coverageestimate(sampsi,k=NA,h_kq2,estimator=kernelest2)

#Long running time!
# The best h-values are found for the 6 distributions, and the coverage probability is found for the KQplc estimator
hp1_KQl<-findbesth(sampsi,p,D1n,kernellevelcross,qnorm(p))
hq1_KQl<-findbesth(sampsi,q,D1n,kernellevelcross,qnorm(q))
hp2_KQl<-findbesth(sampsi,p,D2n,kernellevelcross,qnorm(p))
hq2_KQl<-findbesth(sampsi,q,D2n,kernellevelcross,qnorm(q))
hp3_KQl<-findbesth(sampsi,p,D3n,kernellevelcross,qnorm(p))
hq3_KQl<-findbesth(sampsi,q,D3n,kernellevelcross,qnorm(q))
hp4_KQl<-findbesth(sampsi,p,D4n,kernellevelcross,qnorm(p))
hq4_KQl<-findbesth(sampsi,q,D4n,kernellevelcross,qnorm(q))
hp5_KQl<-findbesth(sampsi,p,D5n,kernellevelcross,qexp(p))
hq5_KQl<-findbesth(sampsi,q,D5n,kernellevelcross,qexp(q))
hp6_KQl<-findbesth(sampsi,p,D6n,kernellevelcross,qlnorm(p))
hq6_KQl<-findbesth(sampsi,q,D6n,kernellevelcross,qlnorm(q))
hkql<-c(hp1_KQl,hq1_KQl,hp2_KQl,hq2_KQl,hp3_KQl,hq3_KQl,hp4_KQl,hq4_KQl,hp5_KQl,hq5_KQl,hp6_KQl,hq6_KQl)
coverageestimate(sampsi,k=NA,hkql,estimator=kernellevelcross)

# The coverage probability is found for the HDplc,SVp1,SVp2, SVp3, NO estimator
coverageestimate(sampsi,k=NA,h=NA,estimator=HDlevelcross)
coverageestimate(sampsi,k=NA,h=NA,estimator=SV1quantile)
coverageestimate(sampsi,k=NA,h=NA,estimator=SV2quantile)
coverageestimate(sampsi,k=NA,h=NA,estimator=SV3quantile)
coverageestimate(sampsi,k=NA,h=NA,estimator=NOquantile)

# end-of-file